<?php
$DATA_DIR = '/tmp/gesa/egide';
$GESA_DIR = '/usr/local/gesa/';

$NMAP_EXEC = '/usr/bin/nmap';
$NMAP_CVE_FILE = 'cve.csv';

$NETCONF_EXEC = $GESA_DIR.'/netconfigure.php';
$NETCONF_SETTINGS = $GESA_DIR.'/public_html/data/network.conf.json';

$longopts = array('debug');
$shortopts = 'D';
$options = getopt($shortopts,$longopts);
$debug_mode = (isset($options['D']) || isset($options['debug']) );
function debug($str)
{
    global $debug_mode;
    if( $debug_mode )
    {
        echo trim($str)."\n";
    }
}

//1 seul scan à la fois
$lock_path = "/tmp/egide_scan.lock";

$fp = fopen($lock_path, 'w+');
if(!flock($fp,LOCK_EX|LOCK_NB)){
	debug("Script is already used\n");
	exit(0);
}

//on récupére tous les scans
$scan_list = scandir($DATA_DIR);
array_shift($scan_list); // dir .
array_shift($scan_list); // dir ..
if( empty($scan_list))
{
    debug("No files to scan\n");
    exit(0);
}

foreach($scan_list as $file)
{
    $task_scan = file_get_contents("$DATA_DIR/$file");
    $task_scan = json_decode($task_scan,TRUE);
    if( $task_scan === FALSE )
    {
        debug("Unable to parse file $DATA_DIR/$file\n");
    }
    if( !isset($task_scan['status'] ) || !isset($task_scan['addresses']) || !isset($task_scan['iface']))
    {
        $message = "Missing data into $DATA_DIR/$file\n";
        debug("$message");
        $task_scan['status'] = 'error';
        $task_scan['message'] = $message;
        file_put_contents("$DATA_DIR/$file",json_encode($task_scan));
        continue;
    }
    if( $task_scan['status'] == 'processing' )
    {
        $message = "Previous scan interrupted and always in processing. Scan state changed to 'error'";
        debug("$message");
        $task_scan['status'] = 'error';
        $task_scan['message'] = $message;
        file_put_contents("$DATA_DIR/$file",json_encode($task_scan));
    }
    if( $task_scan['status'] != 'pending' )
    {
        debug("Ignore $DATA_DIR/$file with status ".$task_scan['status']."\n");
        continue;
    }

    $task_scan['status'] = 'processing';
    $task_scan['start'] = time();
    file_put_contents("$DATA_DIR/$file",json_encode($task_scan));

    $tmpfile = "/tmp/$file";
    $scan_interface = $task_scan['iface'];
    $addresses = str_replace(',',' ',$task_scan['addresses']);

    $iface_conf = get_iface_conf($scan_interface,$NETCONF_SETTINGS);
    if( $iface_conf['vlan'] != 0 )
    {
        $nmap_iface = $iface_conf['name'].'.'.$iface_conf['vlan'];
    }
    else
    {
        $nmap_iface = $iface_conf['name'];
    }

    $enable_ifaces = "$NETCONF_EXEC -f $NETCONF_SETTINGS -t other -n $nmap_iface --enable -i ".$task_scan['addresses'];
    $nmap = "$NMAP_EXEC -e $nmap_iface -sV -oX $tmpfile --min-parallelism=3 --script=vulscan/vulscan.nse --script-args vulscandb=$NMAP_CVE_FILE,\"vulscanoutput='{id};'\""; // il faut rajouter que le  "-e ens18"
    $disable_ifaces = "$NETCONF_EXEC -f $NETCONF_SETTINGS -t other -n $nmap_iface --disable";

    debug("Enable network iface $nmap_iface and route to $addresses");
    exec($enable_ifaces);
    $t0 = date("Y-m-d H:i:s");
    debug("$t0 : Running nmap vulscan $tmpfile $nmap_iface $addresses");
    exec("$nmap $addresses");
    $t1 = date("Y-m-d H:i:s");
    debug("$t1 : Terminated nmap vulscan $tmpfile $nmap_iface $addresses");
    debug("Disable network iface $nmap_iface and network static route");
    exec($disable_ifaces);

    $task_scan['stop'] = time();

    $result = file_get_contents($tmpfile);
    if( $result === FALSE )
    {
        $message = "Unable to read scan output file $tmpfile\n";
        debug("$message");
        $task_scan['status'] = 'error';
        $task_scan['message'] = $message;
        file_put_contents("$DATA_DIR/$file",json_encode($task_scan));
        continue;
    }
    else
    {
        debug("Updating task with output file");
        $task_scan['status'] = 'terminated';
        $task_scan['result'] = $result;
        file_put_contents("$DATA_DIR/$file",json_encode($task_scan));
        unlink($tmpfile);
    }
}

function get_iface_conf($search,$path_file)
{
        $raw_file = file_get_contents($path_file);
        $parsed_file = json_decode($raw_file,TRUE);

        $ret = NULL;
        foreach( $parsed_file as $iface)
        {
                //On cherche par label ou par name.vlan. Le ifname est vraiment en dernier recours
                if( $iface['label'] == $search)
                {
                        return $iface;
                }
                elseif( $iface['name'].'.'.$iface['vlan'] == $search )
                {
                        return $iface;
                }
                elseif( $iface['vlan'] == 0 && $iface['name'] == $search )
                {
                        $ret = $iface;
                }
        }
        return $ret;
}
?>
